function create_selection_tag(tagname, tagcat) { // CLEAR
    let tag = document.createElement("a")
    tag.classList.add("tagselected")
    tag.dataset.tagname = tagname
    tag.dataset.tagcat = tagcat

    tag.insertAdjacentHTML("beforeend", `
        <span class="tag-icon">
            <span class="fa fa-tag"></span>
        </span>
        <span class="content"></span>
        <span class="tag-right">
            <span class="fa fa-close close"></span>
        </span>
    `)
    tag.querySelector("span.content").textContent = tagname.replace(/_/g, ' ')
    // tag.querySelector("input[type=hidden]").setAttribute('name', `tag#${tagname}`)
    return tag
}

// Suggestions
function create_suggestion_tag(tagname, tagcat) { // CLEAR
    let tag = document.createElement("a")
    tag.classList.add("tagsuggestion")
    tag.dataset.tagname = tagname
    tag.dataset.tagcat = tagcat

    tag.insertAdjacentHTML("beforeend", `
        <span class="tag-icon">
            <span class="fa fa-tag"></span>
        </span>
        <span class="content"></span>
        <span class="tag-right">
            <span class="fa fa-plus plus"></span>
        </span>
    `)
    tag.querySelector("span.content").textContent = tagname.replace(/_/g, ' ')
    return tag
}

let taginputs = document.querySelectorAll('.tag-input')
taginputs.forEach(function (tinput) {
    'use strict'

    try {
        let selected = tinput.querySelector('.tags-selected')
        let predef = tinput.querySelector('.tags-inpage')

        let suggestroot = tinput.querySelector('.tag-suggester')

        suggestroot.insertAdjacentHTML("beforeend", `<input type="hidden" name="${suggestroot.dataset.inputname}">`)
        let hidden_input = suggestroot.querySelector(`input[name=${suggestroot.dataset.inputname}][type=hidden]`)

        let suggest_input = suggestroot.querySelector(`input[name=${suggestroot.dataset.inputname}][type=text]`)
        suggest_input.removeAttribute('name')
        if (suggest_input.hasAttribute('required')) {
            suggest_input.removeAttribute('required')
            hidden_input.setAttribute('required', '')
        }

        let suggest_dropdown = suggestroot.querySelector('.suggest-dropdown')


        function hidden_addtag(tagname) { // 1 reference
            let hid_input = hidden_input

            let val = hid_input.value.split(' ')
            val.push(tagname)
            hid_input.value = val.join(' ').trim()
        }
        function hidden_removetag(tagname) { // 1 reference
            let hid_input = hidden_input

            let val = hid_input.value.split(' ')
            val.splice(val.indexOf(tagname), 1)
            hid_input.value = val.join(' ').trim()
        }

        function add_selected(tagname, tagcat) { // 2 references
            let predf = predef
            let sel = selected

            if (predf) {
                let pagetag = predf.querySelector(`a[data-tagname=${tagname}]`)
                if (pagetag) {
                    pagetag.classList.add('tag_hide')
                }
            }

            let newtag = create_selection_tag(tagname, tagcat)
            newtag.addEventListener('click', (event) => {
                remove_selected(event.target.dataset.tagname)
                event.preventDefault()

                setTimeout(() => suggest_input.focus(), 2)
            })

            sel.appendChild(newtag)
            hidden_addtag(tagname)
        }
        function remove_selected(tagname) { // 2 references
            let predf = predef
            let sel = selected

            if (predf) {
                let pagetag = predf.querySelector(`a[data-tagname=${tagname}]`)
                if (pagetag) {
                    pagetag.classList.remove('tag_hide')
                }
            }

            sel.querySelector(`a[data-tagname=${tagname}]`).remove()
            hidden_removetag(tagname)
        }


        // Page tags click to select
        if (predef) {
            predef.querySelectorAll("a").forEach(element => {
                element.addEventListener('click', (event) => {
                    add_selected(event.target.dataset.tagname, event.target.dataset.tagcat)
                    event.preventDefault()

                    setTimeout(() => suggest_input.focus(), 2)
                })
            })
        }

        // Generate selected by url query
        // let query = getJsonFromUrl()
        // if (query['tags']) {
        //     query['tags'].split('+').forEach((tag) => {
        //         add_selected(tag)
        //     })
        // }
        // let vals = suggest_input.value.split(' ')
        // // console.log(vals)
        // vals.forEach(tag => {
        //     if (tag == '') return
        //     add_selected(tag)
        // });
        // Not as clean, but well whatever
        selected.querySelectorAll("a").forEach(element => {
            element.addEventListener('click', (event) => {
                remove_selected(event.target.dataset.tagname)
                event.preventDefault()

                setTimeout(() => suggest_input.focus(), 2)
            })
            hidden_addtag(element.dataset.tagname)
        })
        suggest_input.removeAttribute('value') // Disable prefill by browser

        function select_suggestion(tagelement) {
            add_selected(tagelement.dataset.tagname, tagelement.dataset.tagcat)
            suggest_input.value = ''
            render_suggestions([])

            // yeeeeeeeet
            setTimeout(() => suggest_input.focus(), 2)
        }
        
        function render_suggestions(data) { // 3 references
            let drop = suggest_dropdown
            let sel = selected
            let sug_input = suggest_input

            drop.innerHTML = ''

            let tagnames = Array.from(sel.querySelectorAll('a')).map(a => a.dataset.tagname)
            let i = 0
            data.forEach((el) => {
                if (i > 5) return
                if (tagnames.includes(el.content)) return

                let sugtag = create_suggestion_tag(el.content, el.category.name)

                sugtag.addEventListener('click', (event) => {
                    select_suggestion(event.target)
                    event.preventDefault()
                })

                drop.appendChild(sugtag)

                i++
            });
        }

        // Ajax suggestion handler
        var ajax_timeout
        suggest_input.addEventListener('input', (event) => {
            clearTimeout(ajax_timeout)

            let value = suggest_input.value.replace(/ /g, '_').trim()

            if (value.length < 2) {
                render_suggestions([])
                return
            }

            ajax_timeout = setTimeout(() => {
                fetch('/post/tags?q='+value).then((response) => {
                    console.log(response)
                    return response.json()
                }).then((data) => {
                    // fill suggestions
                    // console.log(data)
                    render_suggestions(data)
                    update_sug_selection('0')
                })
            }, 500)
        })

        let sug_selected = 0
        function update_sug_selection(move) {
            let suggs = suggest_dropdown.querySelectorAll('a.tagsuggestion')
            let last_selected = sug_selected

            let subs = move.substring(0, 1)
            if (subs == '+') {
                sug_selected += parseInt(move.substring(1))
            } else if (subs == '-') {
                sug_selected -= parseInt(move.substring(1))
            } else {
                sug_selected = parseInt(move)
            }

            sug_selected = sug_selected%suggs.length
            if (sug_selected < 0) sug_selected += suggs.length

            try {
                suggs[last_selected].classList.remove('sug-sel')
            } catch (error) {
                // console.log(error)
            }
            try {
                suggs[sug_selected].classList.add('sug-sel')
            } catch (error) {
                // console.log(error)
            }
        }

        suggest_input.addEventListener('keydown', (event) => {
            // console.log(event)
            if (event.keyCode == 13) {
                if (suggest_input.value) {
                    select_suggestion(suggest_dropdown.querySelector('a.tagsuggestion.sug-sel'))
                    event.preventDefault()
                }
            }
            if (event.keyCode == 40 || (event.keyCode == 9 && !event.shiftKey)) {
                update_sug_selection('+1')
                event.preventDefault()
            }
            if (event.keyCode == 38 || (event.keyCode == 9 && event.shiftKey)) {
                update_sug_selection('-1')
                event.preventDefault()
            }
        })

        // search_input.addEventListener('blur', (event) => {
        //     if (event.explicitOriginalTarget != null) {
        //         classlist = event.explicitOriginalTarget.parentNode.classList
        //         if (classlist.contains('tags_selected') || classlist.contains('tags_inpage') || classlist.contains('search_dropdown')) {
        //             setTimeout(() => search_input.focus(), 2)
        //         }
        //     }
        //     console.log(event)
        // })
        // page_tags.querySelectorAll('a').forEach((el) => el.removeAttribute('href'))

    } catch (error) {
        console.log(error)
    }
})




